/*****************************************************************************/
/*
                                  cache.h
*/
/*****************************************************************************/

#ifndef CACHE_H_LOADED
#define CACHE_H_LOADED 1

#include "wasd.h"
#include "file.h"

#define CACHE_ACP_INFO_COUNT_MAX 2048

/**************/
/* structures */
/**************/

#ifdef __ALPHA
#   pragma member_alignment __save
#   pragma member_alignment
#endif

typedef struct CacheStruct FILE_CENTRY;

struct CacheStruct
{
   LIST_ENTRY  CacheListEntry;

   struct CacheStruct  *HashCollisionNextPtr,
                       *HashCollisionPrevPtr;

   BOOL  /* cache is currently being loaded for this path/file */
         CacheLoading,
         /* cache data is currently being revalidated */
         CacheRevalidating,
         /* data load completed OK */
         CacheLoadValid,
         /* is a permanent entry */
         CachePermanent,
         /* cache entry has been invalidated to scavenge memory */
         CacheScavenged,
         /* data in cache is valid and can be use for requests */
         CacheValid,
         /* remove allocated data buffer as soon as not in use */
         Purge,
         /* also remove cache entry */
         PurgeCompletely;

   int  /* the server second where it is still considered recent */
        FrequentTickSecond,
        /* the server second after which it should be revalidated */
        ValidateTickSecond;

   unsigned short /* first free byte in end-of-file VBN */
                  FirstFreeByte,
                  /* file protection bitmap */
                  Protection,
                  /* file ownership UIC group number */
                  UicGroup,
                  /* file ownership UIC member number */
                  UicMember;

   unsigned long  /* size of the block pointed to by 'ContentPtr' */
                  CacheSize,
                  /* number of times current allocated buffer been reused */
                  ContentBufferUsedCount,
                  /* number of times buffer has been reallocated */
                  ContentBufferReallocatedCount,
                  /* length of data (file contents) pointed to 'ContentPtr' */
                  ContentLength,
                  /* virtual block number of end-of-file */
                  EndOfFileVbn,
                  /* number of times cache used */
                  HitCount,
                  /* number of times cache hit but file unmodified */
                  HitNotModifiedCount,
                  /* cache entry is in use and should not be modified */
                  InUseCount,
                  /* number of times the entry has been validated */
                  ValidatedCount;

   unsigned long  /* creation quadword */
                  CdtBinTime [2],
                  /* last hit quadword */
                  HitBinTime [2],
                  /* when cache data loaded quadword */
                  LoadBinTime [2],
                  /* last modified quadword */
                  RdtBinTime [2],
                  /* last validated quadword */
                  ValidateBinTime [2];

   char  /* points to file contents */
         *ContentPtr,
         /* points to file's MIME content type */
         *ContentTypePtr;

   /* MD5 hash of cached resource (can be path, can be file name) */
   MD5_HASH  Md5Hash;

   ODS_STRUCT  FileOds;

   /* used to buffer the function address of the content handler */
   REQUEST_AST  ContentHandlerFunction;
};

#ifdef __ALPHA
#   pragma member_alignment __restore
#endif

/***********************/
/* function prototypes */
/***********************/

CacheAcpInfoAst (REQUEST_STRUCT*);
CacheBegin (REQUEST_STRUCT*);
CacheEnd (REQUEST_STRUCT*);
CacheInit ();
CacheLoadBegin (REQUEST_STRUCT*);
CacheLoadEnd (REQUEST_STRUCT*);
CacheSearch (REQUEST_STRUCT*);
CacheNext (REQUEST_STRUCT*);
CachePurge (BOOL, int*, int*);
CacheReport (REQUEST_STRUCT*, REQUEST_AST, BOOL);
CacheZeroCounters ();

#endif /* CACHE_H_LOADED */

/*****************************************************************************/
